@@ -23,29 +23,28 @@ import soundcard, mpd
 import dataparser
 
 import const, utils
+import random
 import os
 import sys
 import signal
 import cfg
-import intervaltest
+import lessonfile
 
 class Teacher(cfg.ConfigUtils):
-    def __init__(self, exname, app, config):
+    def __init__(self, exname, app):
         cfg.ConfigUtils.__init__(self, exname)
         self.m_app = app
         self.q_status = const.QSTATUS_NO
         self.m_statistics = None
         self.m_timeout_handle = None
-        for key, val in config.iteritems():
-            self.set_string(key, val)
     def get_instrument_config(self, num):
-        if self.get_bool('override_default_instrument'):
-            instr_low = self.get_int('lowest_instrument')
-            instr_low_vel = self.get_int('lowest_instrument_velocity')
-            instr_middle = self.get_int('middle_instrument')
-            instr_middle_vel = self.get_int('middle_instrument_velocity')
-            instr_high = self.get_int('highest_instrument')
-            instr_high_vel = self.get_int('highest_instrument_velocity')
+        if self.get_bool('config/override_default_instrument'):
+            instr_low = self.get_int('config/lowest_instrument')
+            instr_low_vel = self.get_int('config/lowest_instrument_velocity')
+            instr_middle = self.get_int('config/middle_instrument')
+            instr_middle_vel = self.get_int('config/middle_instrument_velocity')
+            instr_high = self.get_int('config/highest_instrument')
+            instr_high_vel = self.get_int('config/highest_instrument_velocity')
         else:
             instr_low = instr_middle = instr_high \
                         = self.get_int('config/preferred_instrument')
@@ -69,11 +68,82 @@ class Teacher(cfg.ConfigUtils):
             self.m_timeout_handle = None
         self.q_status = const.QSTATUS_NO
         soundcard.synth.stop()
-    def configure_exercise(self, collection, lessonfile, D):
-        for k in D:
-            self.set_string(k, D[k])
+        if self.m_statistics:
+            self.m_statistics.save_data()
 
-class MelodicIntervalTeacher(Teacher):
+class LessonbasedTeacher(Teacher):
+    def __init__(self, exname, app):
+        Teacher.__init__(self, exname, app)
+        self.m_P = None
+        self.m_question = None
+    def is_test_complete(self):
+        """
+        Shared between harmonic and melodic interval.
+        """
+        return self.m_test_idx == len(self.m_test_questions) - 1
+    def exit_test_mode(self):
+        """
+        Shared between harmonic and melodic interval.
+        """
+        self.m_statistics.exit_test_mode()
+    def set_lessonfile(self, lessoncollection, lessonfile):
+        """
+        Set the config variables 'lessonfile' and 'lessoncollection'
+        Parse the lesson file and save the statistics.
+        """
+        if lessonfile is not None \
+                and lessonfile == self.get_string('lessonfile'):
+            new_file = 0
+        else:
+            new_file = 1
+        if lessoncollection:
+            self.set_string('lessoncollection', lessoncollection)
+        if lessonfile:
+            self.set_string('lessonfile', lessonfile)
+        self.parse_lessonfile()
+        if self.m_statistics:
+            self.m_statistics.lessonfile_changed(
+             self.get_string('lessoncollection'), self.get_string('lessonfile'))
+    def __make_sure_existing_lessonfile_is_selected(self):
+        if not self.get_string('lessoncollection') or \
+               self.get_string('lessoncollection') \
+                 not in self.m_app.lessonfile_manager.get_collections():
+            self.set_string('lessoncollection', 'solfege')
+        F = self.m_app.lessonfile_manager.get_filenames(
+                 self.get_string('lessoncollection'), self.m_exname)
+        if F:
+            if self.get_string('lessonfile') not in F:
+                self.set_string('lessonfile', F[0])
+        else:
+            self.set_string('lessonfile','')
+    def parse_lessonfile(self):
+        self.m_question = None
+        self.q_status = const.QSTATUS_NO
+        if not self.get_string('lessonfile'):
+            self.m_P = None
+            return
+        fn = self.m_app.lessonfile_manager.get_complete_filename(
+             self.get_string('lessoncollection'),
+             self.get_string('lessonfile'))
+        try:
+            self.m_P = self.lessonfileclass(fn, self)
+        except IOError:
+            self.m_app.m_ui.display_error_message(
+"""There was an IOError while trying to parse a lesson file. The cause was probably you deleting or renaming a lesson file. Please don't do this while Solfege is running. The program is still a little fragile. 
+
+You really should reastart the program now.""")
+        if filter(lambda q: q['music'].m_musictype == 'cmdline',
+                  self.m_P.m_questions):
+            run = self.m_app.m_ui.dialog_yesno(_("The lessonfile contain potentially dangerous code because it run external programs. Run anyway?"))
+            if not run:
+                self.m_P = None
+    def check_askfor(self):
+        if self.get_bool('gui/expert_mode'):
+            self.set_list('ask_for_names', range(len(self.m_P.get_unique_cnames())))
+            for question in self.m_P.m_questions:
+                question['active'] = 1
+
+class MelodicIntervalTeacher(LessonbasedTeacher):
     """
     Base class for interval exercises where
     you can have more than one interval.
@@ -83,8 +153,8 @@ class MelodicIntervalTeacher(Teacher):
     OK = 0
     ERR_PICKY = 1
     ERR_CONFIGURE = 2
-    def __init__(self, exname, app, config):
-        Teacher.__init__(self, exname, app, config)
+    def __init__(self, exname, app):
+        LessonbasedTeacher.__init__(self, exname, app)
         self.m_tonika = None
         self.m_question = []
     def new_question(self, L, H):
@@ -99,6 +169,15 @@ class MelodicIntervalTeacher(Teacher):
             gobject.source_remove(self.m_timeout_handle)
             self.m_timeout_handle = None
 
+        if self.m_app.m_test_mode:
+            self.m_test_idx += 1
+            self.m_question = [self.m_test_questions[self.m_test_idx]]
+            #FIXME use tone pitch range from preferences window.
+            self.m_tonika = mpd.MusicalPitch()
+            self.m_tonika.randomize("f", "f'")
+            self.q_status = const.QSTATUS_NEW
+            return self.OK
+
         if self.get_bool('config/picky_on_new_question') \
               and self.q_status in [const.QSTATUS_NEW, const.QSTATUS_WRONG]:
             return self.ERR_PICKY
@@ -160,114 +239,6 @@ class MelodicIntervalTeacher(Teacher):
                    self.get_int('config/preferred_instrument_velocity'))
         soundcard.synth.play_track(m)
 
-class LessonbasedTeacher(Teacher):
-    def __init__(self, exname, app, lesson_collection, lesson_file, config):
-        Teacher.__init__(self, exname, app, config)
-        self.m_P = None
-        self.m_question = None
-        if lesson_collection or lesson_file:
-            if lesson_collection:
-                self.set_string('lessoncollection', lesson_collection)
-            if lesson_file:
-                self.set_string('lessonfile', lesson_file)
-        self.__make_sure_existing_lessonfile_is_selected()
-    def set_lessonfile(self, lessoncollection, lessonfile):
-        """
-        Set the config variables 'lessonfile' and 'lessoncollection'
-        Parse the lesson file and save the statistics.
-        """
-        if lessonfile is not None \
-                and lessonfile == self.get_string('lessonfile'):
-            new_file = 0
-        else:
-            new_file = 1
-        if lessoncollection:
-            self.set_string('lessoncollection', lessoncollection)
-        if lessonfile:
-            self.set_string('lessonfile', lessonfile)
-        self.__make_sure_existing_lessonfile_is_selected()
-        self.parse_lessonfile()
-        if new_file and self.m_P and self.m_P.header.questions_selectable:
-            self.block_watch('ask_for_names')
-            self.set_list('ask_for_names', range(0, len(self.m_P.get_unique_cnames())))
-            self.unblock_watch('ask_for_names')
-        if self.m_statistics:
-            self.m_statistics.lessonfile_changed(
-             self.get_string('lessoncollection'), self.get_string('lessonfile'))
-    def __make_sure_existing_lessonfile_is_selected(self):
-        if not self.get_string('lessoncollection') or \
-               self.get_string('lessoncollection') \
-                 not in self.m_app.lessonfile_manager.get_collections():
-            self.set_string('lessoncollection', 'solfege')
-        F = self.m_app.lessonfile_manager.get_filenames(
-                 self.get_string('lessoncollection'), self.m_exname)
-        if F:
-            if self.get_string('lessonfile') not in F:
-                self.set_string('lessonfile', F[0])
-        else:
-            self.set_string('lessonfile','')
-    def parse_lessonfile(self):
-        self.m_question = None
-        self.q_status = const.QSTATUS_NO
-        if not self.get_string('lessonfile'):
-            self.m_P = None
-            return
-        fn = self.m_app.lessonfile_manager.get_complete_filename(
-             self.get_string('lessoncollection'),
-             self.get_string('lessonfile'))
-        try:
-            self.m_P = self.lessonfileclass(fn, self)
-        except IOError:
-            self.m_app.m_ui.display_error_message(
-"""There was an IOError while trying to parse a lesson file. The cause was probably you deleting or renaming a lesson file. Please don't do this while Solfege is running. The program is still a little fragile. 
-
-You really should reastart the program now.""")
-        if filter(lambda q: q['music'].m_musictype == 'cmdline',
-                  self.m_P.m_questions):
-            run = self.m_app.m_ui.dialog_yesno(_("The lessonfile contain potentially dangerous code because it run external programs. Run anyway?"))
-            if not run:
-                self.m_P = None
-    def configure_exercise(self, lessoncollection, lessonfile, config):
-        """
-        This function should not be called by the constructors.
-        abstract.Teacher and abstract.LessonbasedTeacher does the same job,
-        but in correct order.
-        """
-        if lessoncollection or lessonfile:
-            if lessoncollection:
-                self.set_string('lessoncollection', lessoncollection)
-            if lessonfile:
-                self.set_string('lessonfile', lessonfile)
-        for key, val in config.iteritems():
-            self.set_string(key, val)
-    def check_askfor(self):
-        """
-        This function will check if the lesson file we have now is the same
-        as the lesson we had last time we used the exercise. If so, the
-        same questions will be activated. If not, all questions in the
-        file will be asked for.
-        """
-        if self.m_P.header.questions_selectable:
-            same_file = self.get_list("ask_for_names_fn") \
-                 == [self.get_string("lessoncollection"),
-                     self.get_string("lessonfile")]
-            if (not self.get_list('ask_for_names')) or (not same_file):
-                self.block_watch('ask_for_names')
-                self.set_list('ask_for_names', range(0, len(self.m_P.get_unique_cnames())))
-                self.set_list("ask_for_names_fn",
-                          [self.get_string("lessoncollection"),
-                           self.get_string("lessonfile")])
-                self.unblock_watch('ask_for_names')
-            for question in self.m_P.m_questions:
-                question['active'] = 0
-            for idx in self.get_list('ask_for_names'):
-                if idx >= len(self.m_P.get_unique_cnames()):
-                    # if we get,  .solfegerc has some bad data.
-                    break
-                cname = self.m_P.get_unique_cnames()[idx]
-                for question in self.m_P.m_questions:
-                    if question.get_cname() == cname:
-                        question['active'] = 1
 
 class Gui(gtk.VBox, cfg.ConfigUtils):
     """Important members:
@@ -291,6 +262,9 @@ class Gui(gtk.VBox, cfg.ConfigUtils):
         self.practise_box = gtk.VBox()
         self.practise_box.show()
         vbox.pack_start(self.practise_box, False)
+        self.g_lesson_heading = gtk.Label()
+        self.practise_box.pack_start(self.g_lesson_heading, padding=18)
+        self.g_lesson_heading.show()
 
         self.action_area = gtk.HBox()
         self.action_area.show()
@@ -311,6 +285,19 @@ class Gui(gtk.VBox, cfg.ConfigUtils):
             self.g_notebook.append_page(self.config_box, gtk.Label(_("Config")))
             self.g_notebook.show()
         self.m_exercise_menu = []
+    def do_test_complete(self):
+        self.on_end_practise()
+        req = self.m_t.m_P.get_test_requirement()
+        res = self.m_t.m_statistics.get()
+        if res >= req:
+            self.g_win.dialog_ok(_("Test completed!\nYour score was %.1f%%.\nThe test requirement was %.1f%%." % (res * 100, req * 100)))
+            self.m_t.m_statistics.store_test_passed()
+        else:
+            self.g_win.dialog_ok(_("Test failed.\nYour score was %.1f%%.\nThe test requirement was %.1f%%." % (res * 100, req * 100)))
+        self.g_win.exit_test_mode()
+    def set_lesson_heading(self, txt):
+        self.g_lesson_heading.set_text('<span size="large"><b>%s</b></span>' % txt)
+        self.g_lesson_heading.set_use_markup(True)
     def on_start_practise(self):
         """Dummy function that child classes can override.
         """
@@ -341,8 +328,6 @@ class Gui(gtk.VBox, cfg.ConfigUtils):
                     self.g_statview.clear()
             else:
                 self.g_statview.update()
-    def get_pretty_name(self):
-        return self.m_t.exercise_data['exercise_pretty_name']
     def _add_auto_new_question_gui(self, box):
         hbox = gu.bHBox(box, False)
         hbox.set_spacing(gnome.ui.PAD_SMALL)
@@ -411,10 +396,8 @@ class IntervalGui(Gui):
                                  inputwidgets.inputwidget_names[i], #new_widget_name,
                                  self.click_on_interval)
         self.practise_box.pack_start(self.g_input)
-        self.practise_box.reorder_child(self.g_input, 0)
+        self.practise_box.reorder_child(self.g_input, 1)
         self.g_input.show()
-        self.m_notenamerange.set_range(self.g_input.m_lowest_tone,
-                                       self.g_input.m_highest_tone)
         if self.m_t.m_tonika:
             # Don't call on_end_practise if we are starting up the exercise.
             # This whole thing is a mess.
@@ -431,34 +414,15 @@ class IntervalGui(Gui):
     def repeat_question(self, *w):
         self.m_t.play_question()
         self.g_input.grab_focus_first_sensitive_button()
-    def show_test_statistics(self, action):
-        if self.g_test_stat_dlg:
-            self.g_test_stat_dlg.destroy()
-            self.g_test_stat_dlg = None
-        t = intervaltest.Test(self.m_t.m_exname)
-        self.g_test_stat_dlg = intervaltest.TestStatisticsDialog(self, _("Test statistics"), t.get_total_results())
-        self.g_test_stat_dlg.show_all()
-        self.g_test_stat_dlg.connect('delete-event', self.destroy_test_stat_dlg)
-    def destroy_test_stat_dlg(self, *ignore):
-        """
-        The self.g_test_stat_dlg is used both for the dialog showing
-        the test statistics and the test results.
-        """
-        assert self.g_test_stat_dlg
-        self.g_test_stat_dlg.destroy()
-        self.g_test_stat_dlg = None
 
 
 class LessonbasedGui(Gui):
     def __init__(self, teacher, window):
         Gui.__init__(self, teacher, window)
-    def get_pretty_name(self):
-        s = self.m_t.exercise_data['exercise_pretty_name']
-        if self.get_string('lessoncollection') == 'solfege':
-            return "%s - %s" % (s, self.get_string('lessonfile'))
-        else:
-            return "%s - %s/%s" % (s,
-             self.get_string('lessoncollection'), self.get_string('lessonfile'))
+    def do_change_lessonfile(self, w=None):
+        self.m_t.set_lessonfile(self.get_string('lessoncollection'),
+            self.get_string('lessonfile'))
+        self.update_gui_after_lessonfile_change()
     def add_random_transpose_gui(self):
         hbox = gu.bHBox(self.config_box, False, False)
         label = gtk.Label(_("Random transpose:"))
