@@ -15,15 +15,6 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-__exercise_data__ = {
-    'exercise_name': 'chord',
-    'stock-icon': 'solfege-chord',
-    'menupath': '/Menubar/FileMenu/ChordsMenu',
-    'name': _('_Chords'),
-}
-import exercise_setup
-exercise_setup.register_exercise(__exercise_data__)
-
 import gtk, gnome
 import gobject
 import gu, widgets
@@ -34,7 +25,6 @@ import soundcard
 import dataparser
 
 class Teacher(abstract.LessonbasedTeacher):
-    exercise_data = __exercise_data__
     OK = 0
     ERR_PICKY = 1
     # valid values for self.q_status:
@@ -44,14 +34,9 @@ class Teacher(abstract.LessonbasedTeacher):
     # QSTATUS_GIVE_UP  after 'Give Up' has been pressed.
     CORRECT = 1
     ALL_CORRECT = 2
-    def __init__(self, exname, app, lesson_collection, lesson_file, config):
-        abstract.LessonbasedTeacher.__init__(self, exname, app,
-              lesson_collection, lesson_file, config)
+    def __init__(self, exname, app):
+        abstract.LessonbasedTeacher.__init__(self, exname, app)
         self.lessonfileclass = lessonfile.ChordLessonfile
-        try: # try: because the file can have changed since startup
-            self.parse_lessonfile()
-        except dataparser.DataparserException, e:
-            self.m_app.m_ui.display_exception_message(e, __file__)
     def new_question(self):
         """
         return OK or ERR_PICKY
@@ -145,11 +130,11 @@ class Gui(abstract.LessonbasedGui):
         hsep = gtk.HSeparator()
         table.attach(hsep, 0, 5, 1, 2, xoptions=gtk.FILL,
                      yoptions=0, ypadding=gnome.ui.PAD_SMALL)
-        vsep = gtk.VSeparator()
-        table.attach(vsep, 1, 2, 0, 4,
+        self.g_vsep1 = gtk.VSeparator()
+        table.attach(self.g_vsep1, 1, 2, 0, 4,
                      xoptions=0, xpadding=gnome.ui.PAD)
-        vsep = gtk.VSeparator()
-        table.attach(vsep, 3, 4, 0, 4,
+        self.g_vsep2 = gtk.VSeparator()
+        table.attach(self.g_vsep2, 3, 4, 0, 4,
                      xoptions=0, xpadding=gnome.ui.PAD)
 
         table.attach(gtk.Label(_("Chord type")), 0, 1, 0, 1,
@@ -157,12 +142,14 @@ class Gui(abstract.LessonbasedGui):
         self.g_chordtype_box = gtk.VBox()
         table.attach(self.g_chordtype_box, 0, 1, 2, 3, xoptions=gtk.FILL)
 
-        table.attach(gtk.Label(_("Inversion")), 2, 3, 0, 1,
+        self.g_inversion_label = gtk.Label(_("Inversion"))
+        table.attach(self.g_inversion_label, 2, 3, 0, 1,
                      xoptions=gtk.FILL, yoptions=0)
         self.g_inversion_box = gtk.VBox()
         table.attach(self.g_inversion_box, 2, 3, 2, 3, xoptions=gtk.FILL)
 
-        table.attach(gtk.Label(_("Toptone")), 4, 5, 0, 1,
+        self.g_toptone_label = gtk.Label(_("Toptone"))
+        table.attach(self.g_toptone_label, 4, 5, 0, 1,
                      xoptions=gtk.FILL, yoptions=0)
         self.g_toptone_box = gtk.VBox()
         table.attach(self.g_toptone_box, 4, 5, 2, 3, xoptions=gtk.FILL)
@@ -189,14 +176,28 @@ class Gui(abstract.LessonbasedGui):
         # config_box #
         ##############
         self.config_box.set_spacing(gnome.ui.PAD_SMALL)
-        self.add_select_lessonfile_gui()
         self.add_random_transpose_gui()
+        # -----------------------------------------
+        self.g_select_questions_category_box, category_box= gu.hig_category_vbox(
+            _("Chord types to ask"))
+        self.config_box.pack_start(self.g_select_questions_category_box, True)
+        self.g_select_questions = widgets.QuestionNameCheckButtonTable(self.m_t)
+        self.g_select_questions.initialize(4, 0)
+        category_box.pack_start(self.g_select_questions, False)
+        self.g_select_questions.show()
 
-        self.g_instrument_configurator  \
-              = widgets.InstrumentConfigurator(self.m_exname, 3)
-        self.config_box.pack_start(self.g_instrument_configurator, False)
-        self.g_instrument_configurator.show()
         self.update_gui_after_lessonfile_change()
+    def update_select_question_buttons(self):
+        if self.get_bool('gui/expert_mode'):
+            self.g_select_questions_category_box.show()
+            self.g_select_questions.initialize(self.m_t.m_P.header.fillnum,
+                                 self.m_t.m_P.header.filldir)
+            self.m_t.check_askfor()
+            for question in self.m_t.m_P.iterate_questions_with_unique_names():
+                self.g_select_questions.add(question, 'normal')
+        else:
+            self.g_select_questions_category_box.hide()
+            self.g_select_questions.initialize(0, 0)
     def update_answer_buttons(self, obj=None):
         for x in self.g_chordtype_box.get_children() \
               + self.g_inversion_box.get_children() \
@@ -239,15 +240,28 @@ class Gui(abstract.LessonbasedGui):
     def update_gui_after_lessonfile_change(self):
         self.g_music_displayer.clear()
         if self.m_t.m_P:
+            self.update_select_question_buttons()
+            self.set_lesson_heading(self.m_t.m_P.header.lesson_heading)
             self.g_new.set_sensitive(True)
             self.g_random_transpose.set_text(str(self.m_t.m_P.header.random_transpose))
+            if not [q for q in self.m_t.m_P.m_questions if 'toptone' in q]:
+                self.g_toptone_label.hide()
+                self.g_vsep2.hide()
+            else:
+                self.g_toptone_label.show()
+                self.g_vsep2.show()
+            if not [q for q in self.m_t.m_P.m_questions if 'inversion' in q]:
+                self.g_inversion_label.hide()
+                self.g_vsep1.hide()
+            else:
+                self.g_inversion_label.show()
+                self.g_vsep1.show()
         else:
             self.g_new.set_sensitive(False)
         self.g_repeat.set_sensitive(False)
         self.g_repeat_arpeggio.set_sensitive(False)
         self.g_give_up.set_sensitive(False)
         self.update_answer_buttons()
-        self.g_win.set_title("Solfege - " + self.get_pretty_name())
     def on_type(self, button, event=None):
         if event is None:
             self.on_type_left_clicked(button)
@@ -447,6 +461,10 @@ class Gui(abstract.LessonbasedGui):
             self.g_win.display_question_music_error_message(
                 self.m_t.m_P._idx, self.get_string('lessonfile'), e)
     def on_start_practise(self):
+        self.g_flashbar.require_size([
+            _("Click 'New chord' to begin."),
+            _("XXXX, root position, toptone: 5"),
+        ])
         self.g_new.grab_focus()
         gobject.timeout_add(const.SHORT_WAIT, lambda self=self:
             self.g_flashbar.flash(_("Click 'New chord' to begin.")))
@@ -467,10 +485,24 @@ class Gui(abstract.LessonbasedGui):
                 inversion_txt = _("root position")
             elif i > 0:
                 inversion_txt = _("%i. inversion") % i
+            else:
+                inversion_txt = ""
             i = self.m_t.m_P.get_toptone()
             if i != -1:
                 toptone_txt = "toptone: %i" % i
-            self.g_flashbar.flash(_("%s, %s and %s") % (type_txt, inversion_txt, toptone_txt))
+            else:
+                toptone_txt = ""
+            v = [type_txt]
+            if inversion_txt:
+                v.append(inversion_txt)
+            if toptone_txt:
+                v.append(toptone_txt)
+            if len(v) == 3:
+                self.g_flashbar.flash(_("%s, %s and %s") % tuple(v))
+            elif len(v) == 2:
+                self.g_flashbar.flash(_("%s and %s") % tuple(v))
+            else:
+                self.g_flashbar.flash(type_txt)
             self.g_new.set_sensitive(True)
             self.g_give_up.set_sensitive(False)
             for button in self.g_chordtype_box.get_children():
