@@ -15,15 +15,6 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-__exercise_data__ = {
-    'exercise_name': 'compare-intervals',
-    'stock-icon': 'solfege-compare-intervals',
-    'menupath': '/Menubar/FileMenu/IntervalsMenu',
-    'name': _("C_ompare intervals"),
-}
-import exercise_setup
-exercise_setup.register_exercise(__exercise_data__)
-
 import gtk, gnome.ui
 import gobject
 import random
@@ -31,15 +22,16 @@ import random
 import gu, widgets, utils
 import mpd, mpd.musicdisplayer, soundcard
 import abstract, const
+import lessonfile
 
 
-class Teacher(abstract.Teacher):
-    exercise_data = __exercise_data__
+class Teacher(abstract.LessonbasedTeacher):
     OK = 0
     ERR_PICKY = 1
     ERR_CONFIGURE = 2
-    def __init__(self, exname, app, config):
-        abstract.Teacher.__init__(self, exname, app, config)
+    def __init__(self, exname, app):
+        abstract.LessonbasedTeacher.__init__(self, exname, app)
+        self.lessonfileclass = lessonfile.IdByNameLessonfile
     def give_up(self):
         self.q_status = const.QSTATUS_GIVE_UP
     def new_question(self):
@@ -164,7 +156,8 @@ class Gui(abstract.Gui):
         # config_box #
         ##############
         self.g_instrument_configurator \
-                 = widgets.InstrumentConfigurator(self.m_exname, 2)
+                 = widgets.InstrumentConfigurator(self.m_exname, 2,
+                    _("_Change midi instrument for this exercise"))
         self.config_box.pack_start(self.g_instrument_configurator, False)
         # ------------------------------------------
         self._add_auto_new_question_gui(self.config_box)
@@ -186,30 +179,37 @@ class Gui(abstract.Gui):
             box.pack_start(b, False)
             return D
         #---------
-        hbox = gu.bHBox(self.config_box, False)
+        self.g_intervalconfig_box = gu.bVBox(self.config_box, False)
+        hbox = gu.bHBox(self.g_intervalconfig_box, False)
         hbox.pack_start(gtk.Label(_("First interval:")), False,
                         padding=gnome.ui.PAD_SMALL)
         self.g_rdbs = [pack_rdbs(hbox, self.update_first)]
         self.g_first_interval_up = widgets.IntervalCheckBox(self.m_exname,
                                                          'first_interval_up')
-        self.config_box.pack_start(self.g_first_interval_up, False)
+        self.g_intervalconfig_box.pack_start(self.g_first_interval_up, False)
         self.g_first_interval_down = widgets.IntervalCheckBox(self.m_exname,
                              'first_interval_down')
-        self.config_box.pack_start(self.g_first_interval_down, False)
+        self.g_intervalconfig_box.pack_start(self.g_first_interval_down, False)
         #----------
-        hbox = gu.bHBox(self.config_box, False)
+        hbox = gu.bHBox(self.g_intervalconfig_box, False)
         hbox.pack_start(gtk.Label(_("Last interval:")), False,
                         padding=gnome.ui.PAD_SMALL)
         self.g_rdbs.append (pack_rdbs(hbox, self.update_last))
         self.g_last_interval_up = widgets.IntervalCheckBox(self.m_exname,
                      'last_interval_up')
-        self.config_box.pack_start(self.g_last_interval_up, False)
+        self.g_intervalconfig_box.pack_start(self.g_last_interval_up, False)
         self.g_last_interval_down = widgets.IntervalCheckBox(self.m_exname,
                         'last_interval_down')
-        self.config_box.pack_start(self.g_last_interval_down, False)
+        self.g_intervalconfig_box.pack_start(self.g_last_interval_down, False)
         #------------
+        s = self.get_string('first_interval_type')
+        if not s in ('harmonic', 'up', 'down', 'up/down'):
+            self.set_string('first_interval_type', 'harmonic')
         self.g_rdbs[0][self.get_string('first_interval_type')].set_active(True)
         self.update_first(self.g_rdbs[0][self.get_string('first_interval_type')])
+        s = self.get_string('last_interval_type')
+        if not s in ('harmonic', 'up', 'down', 'up/down'):
+            self.set_string('last_interval_type', 'harmonic')
         self.g_rdbs[1][self.get_string('last_interval_type')].set_active(True)
         self.update_last(self.g_rdbs[1][self.get_string('last_interval_type')])
         self.config_box.show_all()
@@ -326,6 +326,10 @@ class Gui(abstract.Gui):
         gobject.timeout_add(const.SHORT_WAIT, lambda self=self:
             self.g_flashbar.flash(_("Click 'New' to begin.")))
         self.g_new.grab_focus()
+        if self.get_bool('gui/expert_mode'):
+            self.g_intervalconfig_box.show()
+        else:
+            self.g_intervalconfig_box.hide()
     def on_end_practise(self):
         self.m_t.end_practise()
         self.g_first_is_largest.set_sensitive(False)
