@@ -51,7 +51,7 @@ if __name__ == "__main__" or __name__ == "dataparser":
 import os, os.path
 import re
 
-tokens = ('NAME', 'STRING', 'OPERATOR', 'INTEGER', 'CHAR', 'EOF')
+tokens = ('NAME', 'STRING', 'OPERATOR', 'INTEGER', 'FLOAT', 'CHAR', 'EOF')
 for t in tokens:
     globals()[t] = t
 del t
@@ -59,6 +59,7 @@ del t
 NEW_re = re.compile("""(?:
                         (\s+)|  #space
                         (\#.*?$)| #comment
+                        (-?\d+\.\d+) | #float
                         (-?\d+)| #integer
                         (\"\"\"(.*?)\"\"\")| #multiline string
                         ("(.*?)")| #string
@@ -67,21 +68,24 @@ NEW_re = re.compile("""(?:
                       re.VERBOSE|re.MULTILINE|re.DOTALL|re.UNICODE)
 
 LI_INTEGER = NEW_re.match("-3").lastindex
+LI_FLOAT = NEW_re.match("3.3").lastindex
 LI_MSTRING = NEW_re.match('"""string"""').lastindex
 LI_STRING = NEW_re.match('"string"').lastindex
 LI_NAME = NEW_re.match("name").lastindex
 LI_COMMENT = NEW_re.match("# comment").lastindex
 
 lastindex_to_ID = {LI_INTEGER: INTEGER,
-                     LI_STRING: STRING,
+                     LI_FLOAT: FLOAT,
+                    LI_STRING: STRING,
                      LI_MSTRING: STRING,
                      LI_NAME: NAME,
                     }
 
-lastindex_to_group = {LI_INTEGER: 3,
-                     LI_STRING: 7,
-                     LI_MSTRING: 5,
-                     LI_NAME: 8,
+lastindex_to_group = {LI_INTEGER: 4,
+                     LI_STRING: 8,
+                     LI_MSTRING: 6,
+                     LI_NAME: 9,
+                     LI_FLOAT: 3,
                     }
 
 class _TranslatedString:
@@ -319,7 +323,8 @@ class Dataparser:
         ifile.close()
         self.prog()
         self._lexer = old_lexer
-        self.header = old_header
+        for k, v in old_header.items():
+            self.header[k] = v
         self._lexer.scan(')')
     def assignmentlist(self):
         """assignmentlist: (assignment+) """
@@ -375,12 +380,14 @@ class Dataparser:
             faktorlist.append(self.faktor())
         return faktorlist
     def atom(self):
-        """atom: INTEGER | STRING | NAME | FUNCTIONCALL"""
+        """atom: INTEGER | FLOAT | STRING | NAME | FUNCTIONCALL"""
         peek = self._lexer.peek_type()
         if peek == 'STRING':
             return self._lexer.scan('STRING')
         elif peek == 'INTEGER':
             return int(self._lexer.scan('INTEGER'))
+        elif peek == 'FLOAT':
+            return float(self._lexer.scan('FLOAT'))
         elif peek == 'NAME':
             if self._lexer.peek_type(1) == '(':
                 return self.functioncall()
@@ -516,11 +523,11 @@ def main():
             p = Dataparser({'dictation': 'dictation',
                   'progression': 'progression',
                   'harmony': 'harmony',
-                  'sing-chord': 'sing-chord',
+                  'singchord': 'singchord',
                   'singanswer': 'singanswer',
-                  'chord-voicing': 'chord-voicing',
+                  'chordvoicing': 'chordvoicing',
                   'chord': 'chord',
-                  'id-by-name': 'id-by-name',
+                  'idbyname': 'idbyname',
                   'satb': 'satb',
                   'horiz': 'horiz',
                   'vertic': 'vertic',
