@@ -181,20 +181,21 @@ class FlashBar(gtk.Frame):
         self.__label = widgets.HarmonicProgressionLabel('')
         self.add(self.__label)
         self.__timeout = None
+        # The allocated size
+        self.m_sx, self.m_sy = 0, 0
     def require_size(self, stringlist):
         """
         stringlist is a list of the strings believed to be widest.
         require_size will make sure that the flashbar is at least large enough to
         show all the strings in stringlist.
         """
-        mx, my = 0, 0
         for s in stringlist:
             self.__label.set_text(s)
             x, y =  self.__label.size_request()
-            mx = max(x, mx)
-            my = max(y, my)
+            self.m_sx = max(x, self.m_sx)
+            self.m_sy = max(y, self.m_sy)
         self.__label.set_text("")
-        self.__label.set_size_request(mx, my)
+        self.__label.set_size_request(self.m_sx, self.m_sy)
     def flash(self, txt):
         """Display a message that is automatically removed after some time.
         If we flash a new message before the old flashed message are removed,
@@ -202,7 +203,12 @@ class FlashBar(gtk.Frame):
         """
         if self.__timeout:
             gobject.source_remove(self.__timeout)
+        self.__label.set_size_request(-1, -1)
         self.__label.set_text(txt)
+        sx, sy = self.__label.size_request()
+        self.m_sx = max(sx, self.m_sx)
+        self.m_sy = max(sy, self.m_sy)
+        self.__label.set_size_request(self.m_sx, self.m_sy)
         def f(self=self):
             self.__timeout = None
             if self.__stack:
