@@ -260,6 +260,11 @@ _("""You have to select some tones practise. Do this on the config page by setti
     def set_percentage_label(self):
         self.g_percentage.set_text("%.1f %%" % (self.m_t.m_statistics.get()*100))
     def on_start_practise(self):
+        self.g_flashbar.require_size([
+            _("Click 'New tone' to begin."),
+            _("Correct, but you have already solved this question"),
+            _("Wrong, but you have already solved this question"),
+        ])
         if not runtime.has_gnome_canvas():
             return # this exercise don't work without GnomeCanvas
         for n in ('c', 'cis', 'd', 'dis', 'e', 'f', 'fis', 'g', 'gis', 'a',
