@@ -91,6 +91,7 @@ _keys_to_interval = {
             }
 
 predef = {'dictation': 'dictation',
+          'rhythm': 'rhythm',
           'harmonicprogressiondictation': 'harmonicprogressiondictation',
           'singchord': 'singchord',
           'singanswer': 'singanswer',
@@ -153,6 +154,13 @@ class _Header:
                            ('have_music_displayer', False),
                            ('enable_right_click', True),
                            ('disable_unused_intervals', True),
+                           # use only by the rhythm_exercise
+                           ('num_beats', 4.0),
+                           ('bpm', 60),
+                           ('countin_perc', 75),
+                           ('rhythm_perc', 37),
+                           ('count_in', 2.0),
+                           ('not_start_with_rest', True),
                            ):
             if key not in self.m_headerdict:
                 self.m_headerdict[key] = value
@@ -859,7 +867,7 @@ class LessonFileManager:
                     if not f.has_lesson_id():
                         f.add_lesson_id()
                 except IOError, e:
-                    print >> sys.stderr, "add_lesson_id: Failed because file is write protected: '%s'" % self.get_complete_filename(collection, filename)
+                    print >> sys.stderr, "add_lesson_id('%s'):" % self.get_complete_filename(collection, filename), e
                     continue
                 try:
                     p = dataparser.Dataparser(predef, lessonfile_functions)
