@@ -161,6 +161,8 @@ import mpd.musicdisplayer
 import gu
 import cfg
 import stock
+import lessonfile
+import abstract
 
 import chord
 import chordvoicing
@@ -168,16 +170,6 @@ import harmonicinterval
 import melodicinterval
 import singinterval
 
-import exercise_setup
-exercise_setup.register_exercise({
-    'exercise_name': 'id-by-name',
-    'stock-icon': 'solfege-identify-scale',
-    'menupath': '/Menubar/FileMenu/MiscMenu',
-    'name': _("_Scales"),
-    'exercise_config': 'solfege/scales',
-    'action': 'start-exercise-scales',
-})
-
 import idbyname
 import dictation
 import twelvetone
@@ -189,17 +181,7 @@ import identifybpm
 import harmonicprogressiondictation
 import singanswer
 #import example
-
-exercise_setup.register_exercise({
-    'exercise_name': 'id-by-name',
-    'stock-icon': 'solfege-identify-scale',
-    'menupath': '/Menubar/FileMenu/MiscMenu',
-    'name': _("Scales in t_hirds"),
-    'exercise_config': 'solfege/scales-thirds',
-    'action': 'start-exercise-scales-in-thirds',
-    'notoolbar': True,
-})
-
+import learning_tree_editor
 
 class DocViewer(gtk.VBox):
     def __init__(self, activate_cb):
@@ -208,7 +190,6 @@ class DocViewer(gtk.VBox):
         self.m_htmlwidget.show_all()
         self.pack_start(self.m_htmlwidget)
         self.set_size_request(500, 300)
-        #self.read_docfile = self.m_htmlwidget.read_docfile
         self.get_vadjustment = self.m_htmlwidget.get_vadjustment
         self.on_key_press_event = self.m_htmlwidget.on_key_press_event
         self.m_statusbar = gtk.Statusbar()
@@ -223,13 +204,8 @@ class DocViewer(gtk.VBox):
         """
         if fn:
             self.m_htmlwidget.read_docfile(fn)
-        #if anchor:
-        #    #FIXME jump_to_anchor is not implemented yet it gnome-python,
-        #    # or maybe I just don't understand how to use it.
-        #    self.m_htmlwidget.g_view.jump_to_anchor(anchor)
-        #else:
-        a = self.m_htmlwidget.get_vadjustment()
-        a.set_value(0.0)
+        if anchor and 'g_view' in dir(self.m_htmlwidget):
+            self.m_htmlwidget.g_view.jump_to_anchor(anchor)
     def on_anchor_track(self, url):
         if url:
             # remove newlines in url because it make the window resize
@@ -239,12 +215,40 @@ class DocViewer(gtk.VBox):
         else:
             self.m_statusbar.pop(1)
 
+class ResolveLessonIdCrashDlg(gtk.Dialog):
+    def __init__(self, lessonfile_manager, lesson_id):
+        gtk.Dialog.__init__(self, _("Resolve lesson_id crash"), None,
+            buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
+                     gtk.STOCK_OK, gtk.RESPONSE_OK))
+        vbox = gtk.VBox()
+        vbox.set_border_width(12)
+        self.vbox.pack_start(vbox)
+        msg1 = gtk.Label("<b>%s</b>" % _("Lesson_id crash"))
+        msg1.set_alignment(0.0, 0.5)
+        msg1.set_use_markup(True)
+        vbox.pack_start(msg1, False)
+        msg2 = gtk.Label(_("The lesson_id has to be unique for each lesson file. Please select which lesson file shall keep the current lesson_id. The preselected file is Solfege's educated guess. Press 'Cancel' to postpone the decision."))
+        msg2.set_alignment(0.0, 0.5)
+        msg2.set_line_wrap(True)
+        vbox.pack_start(msg2)
+        self.radio_dict = {}
+        for info in lessonfile_manager.get_lesson_file_info(lesson_id):
+            if self.radio_dict.keys():
+                g = self.radio_dict[self.radio_dict.keys()[0]]
+            else:
+                g = None
+            radiobutton = gtk.RadioButton(g, "%s %s" % (info['filename'], info['timestr']))
+            radiobutton.set_data('fn', info['filename'])
+            self.radio_dict[info['mtime']] = radiobutton
+            vbox.pack_start(radiobutton)
+        self.radio_dict[min(self.radio_dict.keys())].set_active(True)
+        self.show_all()
 
 class MusicViewerWindow(gtk.Dialog):
     def __init__(self, app):
         gtk.Dialog.__init__(self)
         self.m_app = app
-        self.set_default_size(400, 200)
+        self.set_default_size(500, 300)
         self.g_music_displayer = mpd.musicdisplayer.MusicDisplayer(utils.play_tone)
         self.vbox.pack_start(self.g_music_displayer)
         b = gu.bButton(self.action_area, _("Close"), self.m_app.close_musicviewer)
@@ -256,7 +260,7 @@ class MusicViewerWindow(gtk.Dialog):
         self.g_music_displayer.display(music, fontsize)
 
 class MainWin(gnome.ui.App, cfg.ConfigUtils):
-    def __init__(self, options, datadir):
+    def __init__(self, options, datadir, lessonfile_manager):
         gnome.ui.App.__init__(self, 'solfege', 'GNU Solfege')
         self.icons = stock.SolfegeIconFactory(self, datadir)
         self.set_default_size(400, 400)
@@ -280,21 +284,38 @@ class MainWin(gnome.ui.App, cfg.ConfigUtils):
         self.m_key_bindings = {'history_back_ak': self.history_back,
                                'history_forward_ak': self.history_forward,
                                'history_reload_ak': self.history_reload}
-        self.m_app = app.SolfegeApp(options, self)
-        self.setup_menu_and_toolbar()
+        self.m_app = app.SolfegeApp(options, self, lessonfile_manager)
+        self.g_ui_manager = gtk.UIManager()
+        self.m_tree = learning_tree_editor.LearningTree(self.m_app.lessonfile_manager)
+        self.m_tree.load('learningtree.txt')
+        self.m_action_groups = {
+            'Practise': gtk.ActionGroup('Practise'),
+            'Exit': gtk.ActionGroup('Exit'),
+            'NotExit': gtk.ActionGroup('NotExit'),
+        }
+        for a in self.m_action_groups.values():
+            self.g_ui_manager.insert_action_group(a, 1)
+        self.setup_menu()
         self.m_app.handle_href('welcome.html')
-    def setup_menu_and_toolbar(self):
-        self.g_action_group = gtk.ActionGroup('SolfegeActions')
-        self.g_action_group.add_actions([
+    def on_learning_tree_changed(self, *v):
+        self.m_tree = learning_tree_editor.LearningTree(self.m_app.lessonfile_manager)
+        self.m_tree.load('learningtree.txt')
+        self.create_practise_and_test_menu()
+    def setup_menu(self):
+        self.m_action_groups['Exit'].add_actions([
           ('FileMenu', None, _('_File')),
-          ('ChordsMenu', None, _('_Chords')),
-          ('IntervalsMenu', None, _('_Intervals')),
-          ('RhythmMenu', None, _('_Rhythm')),
+          ('AppQuit', 'gtk-quit', None, None, None, self.quit_program),
+        ])
+        self.m_action_groups['NotExit'].add_actions([
           ('MiscMenu', None, _('_Misc exercises')),
           ('TheoryMenu', None, _('_Theory')),
           ('TheoryIntervals', None, _('_Intervals'), None, None,
             lambda o: self.m_app.handle_href('theory-intervals.html')),
-          ('ExerciseMenu', None, ''),
+          ('EditMenu', None, _("_Edit")),
+          ('TreeEditor', None, _('Learning tree'), None, None,
+            self.do_tree_editor),
+          ('PractiseMenu', None, _("_Practise")),
+          ('TestsMenu', None, _("_Tests")),
           ('ViewMenu', None, _('_View')),
           ('OpenPreferencesWindow', 'gtk-preferences', None, '<ctrl>F12', None,
             self.open_preferences_window),
@@ -312,7 +333,6 @@ class MainWin(gnome.ui.App, cfg.ConfigUtils):
           ('HelpReportingBugs', None, _('Reporting _bugs'), None, None,
             lambda o: self.m_app.handle_href('bug-reporting.html')),
           ('HelpAbout', 'gtk-about', None, None, None, self.show_about_window),
-          ('AppQuit', 'gtk-quit', None, None, None, self.quit_program),
           ('ViewZoomIn', 'gtk-zoom-in', _('Help viewer zoom in'), '<ctrl>plus', None,
            lambda w: self.box_dict['docviewer'].m_htmlwidget.g_view.zoom_in()),
           ('ViewZoomOut', 'gtk-zoom-out', _('Help viewer zoom out'), '<ctrl>minus', None,
@@ -324,25 +344,9 @@ class MainWin(gnome.ui.App, cfg.ConfigUtils):
           ('ShowBugReports', None, _('See your bug reports'), None, None,
             self.show_bug_reports),
         ])
-        self.g_ui_manager = gtk.UIManager()
-        self.g_ui_manager.insert_action_group(self.g_action_group, 1)
 
         self.g_ui_manager.add_ui_from_file("ui.xml")
 
-        a = gtk.ToggleAction('ToggleNavButtonsVisible', _('_Navigation buttons'), None, None)
-        if self.get_bool('gui/navbar_visible'):
-            a.set_active(True)
-        self.add_watch('gui/navbar_visible', self.on_watch_navbar_visible)
-        a.connect('activate', lambda w: self.set_bool('gui/navbar_visible', w.get_active()))
-        self.g_action_group.add_action(a)
-
-        a = gtk.ToggleAction('ToggleToolbarVisible', _('_Toolbar'), None, None)
-        if self.get_bool('gui/toolbar_visible'):
-            a.set_active(True)
-        self.add_watch('gui/toolbar_visible', self.on_watch_toolbar_visible)
-        a.connect('activate', lambda w: self.set_bool('gui/toolbar_visible', w.get_active()))
-        self.g_action_group.add_action(a)
-
         if runtime.has_gtkhtml2():
             self.g_ui_manager.add_ui_from_string("""\
 <menubar name='Menubar'>
@@ -366,31 +370,76 @@ class MainWin(gnome.ui.App, cfg.ConfigUtils):
         for e in __exdata__:
             if 'menupath' in e:
                 a = gtk.Action(e['action'], e['name'], e['exercise_pretty_name'], e['stock-icon'])
-                self.g_action_group.add_action(a)
+                self.m_action_groups['NotExit'].add_action(a)
                 a.connect('activate', lambda o, e=e:
                     self.m_app.handle_href('solfege:/%s/%s' % (
                       e['exercise_name'], e['exercise_config'])))
                 if not self.g_ui_manager.get_widget(e['menupath']):
-                    print "Miss:", e['menupath']
+                    print "Miss:", e['menupath'], e['exercise_name']
                 self.g_ui_manager.add_ui(self.g_ui_manager.new_merge_id(),
                    e['menupath'], e['name'], e['action'],
                    gtk.UI_MANAGER_MENUITEM, False)
-                if 'notoolbar' not in e:
-                    self.g_ui_manager.add_ui(self.g_ui_manager.new_merge_id(),
-                       '/Toolbar', e['name'], e['action'],
-                       gtk.UI_MANAGER_TOOLITEM, False)
         self.set_menus(self.g_ui_manager.get_widget('/Menubar'))
-        self.on_watch_toolbar_style('gui/toolbar_style')
-        self.add_watch('gui/toolbar_style', self.on_watch_toolbar_style)
-        self.main_box.pack_start(self.g_ui_manager.get_widget('/Toolbar'), False)
-        if not self.get_bool('gui/toolbar_visible'):
-            self.g_ui_manager.get_widget('/Toolbar').hide()
-        if not self.get_bool('gui/navbar_visible'):
-            self.g_ui_manager.get_widget('/Toolbar/BrowserGoBack').hide()
-            self.g_ui_manager.get_widget('/Toolbar/BrowserGoForward').hide()
-        #FIXME not using GnomeApp.add_toolbar
-        #self.add_toolbar(self.g_ui_manager.get_widget('/Toolbar'),
-        #  'toolbaren', 'normal', 'top', 1, 0,0)
+        self.m_help_on_current_merge_id = None
+    def create_practise_and_test_menu(self):
+        """
+        Create the Practise menu.
+        This function has to be called after we have cleared out any
+        lesson_id crashes.
+        """
+        actions = []
+        if self.m_practise_and_test_merge_id:
+            self.g_ui_manager.remove_ui(self.m_practise_and_test_merge_id)
+        s = """
+<menubar name='Menubar'>
+ <menu action='PractiseMenu'>"""
+        tests = "<menu action='TestsMenu'>"
+        for topic in self.m_tree.m_topics:
+            s += "<menu action='%s'>" % topic['name']
+            tests += "<menu action='TEST_%s'>" % topic['name']
+            actions.append((topic['name'], None, topic['name']))
+            actions.append(('TEST_%s' % topic['name'], None, topic['name']))
+            for lesson_id in topic['lessons']:
+                if self.m_tree.m_visibilities[lesson_id] > self.m_tree.m_visibility:
+                    continue
+                menutitle = self.m_app.lessonfile_manager.m_uiddb[lesson_id]['header']['title']
+                actions.append((
+                    lesson_id, None, menutitle, None, None,
+                    lambda o, e=lesson_id: self.m_app.practise_lesson_id(e)))
+                if self.m_app.lessonfile_manager.is_test_passed(lesson_id):
+                    menutitle = _("%s (passed)") % menutitle
+                if self.m_app.lessonfile_manager.get(lesson_id, 'test'):
+                    actions.append((
+                        'TEST_%s' % lesson_id, None,
+                        menutitle,
+                        None, None,
+                        lambda o, e=lesson_id: self.m_app.test_lesson_id(e)))
+                    tests += "<menuitem action='TEST_%s' />" % lesson_id
+                s += "<menuitem action='%s' />" % lesson_id
+            s += "</menu>"
+            tests += "</menu>"
+        self.m_action_groups['NotExit'].add_actions(actions)
+        s += "</menu>"
+        tests += "</menu>"
+        s += tests
+        s += "</menubar>"
+        self.m_practise_and_test_merge_id = self.g_ui_manager.add_ui_from_string(s)
+    def show_help_on_current(self):
+        #assert self.m_help_on_current_merge_id == None
+        if self.m_help_on_current_merge_id:
+            # Already visible
+            return
+        self.m_help_on_current_merge_id = self.g_ui_manager.add_ui_from_string("""
+<menubar name='Menubar'>
+  <menu action='HelpMenu'>
+    <menuitem position='top' action='HelpHelp' />
+  </menu>
+</menubar>""")
+    def hide_help_on_current(self):
+        if not self.m_help_on_current_merge_id:
+            return
+        self.g_ui_manager.remove_ui(self.m_help_on_current_merge_id)
+        self.m_help_on_current_merge_id = None
     def dialog_ok(self, text):
         """"
         Return the gtk.RESPONSE_XXXX returned by .run()
@@ -489,25 +538,37 @@ class MainWin(gnome.ui.App, cfg.ConfigUtils):
     def hide_about_window(self, widget):
         self.g_about_window.destroy()
         self.g_about_window = None
+    def do_tree_editor(self, *v):
+        win = learning_tree_editor.Window(self.m_app)
+        win.load_file('learningtree.txt')
+        win.show()
+        x, y = win.g_topic_scrollwin.size_request()
+        win.g_lesson_scrollwin.set_size_request(x, int(y*1.6))
+        win.g_deps_scrollwin.set_size_request(x, int(y*1.6))
     def post_constructor(self):
+        self.m_practise_and_test_merge_id = None
+        self.create_practise_and_test_menu()
         self.box_dict['docviewer'].grab_focus()
         if self.m_app.m_sound_init_exception is not None:
             self.m_app.display_sound_init_error_message(self.m_app.m_sound_init_exception)
-    def activate_exercise(self, href):
-        protocol, action, exname, anchor, collection, lessonfile, config = \
-           utils.parse_url(href)
-        assert protocol == 'solfege'
+    def activate_exercise(self, module, urlobj=None):
         if self.m_viewer:
             self.box_dict[self.m_viewer].hide()
-        self.box_dict[exname].show()
-        self.m_viewer = exname
-        if action in ['practise', 'config', 'statistics']:
-            # We need this test because not all exercises use a notebook.
-            if self.box_dict[self.m_viewer].g_notebook:
+        self.m_viewer = module
+        self.box_dict[module].show()
+        # We need this test because not all exercises use a notebook.
+        if self.box_dict[self.m_viewer].g_notebook:
+            if urlobj and urlobj.action in ['practise', 'config', 'statistics']:
                 self.box_dict[self.m_viewer].g_notebook.set_current_page(
-                   ['practise', 'config', 'statistics'].index(action))
-        self.set_title("Solfege - " + self.box_dict[exname].get_pretty_name())
-        self.m_history.add(href)
+                   ['practise', 'config', 'statistics'].index(urlobj.action))
+            else:
+                self.box_dict[self.m_viewer].g_notebook.set_current_page(0)
+        if isinstance(self.box_dict[module].m_t, abstract.LessonbasedTeacher):
+            self.set_title("Solfege - " + self.box_dict[module].m_t.m_P.header.title)
+        else:
+            self.set_title("Solfege - " + self.box_dict[module].m_t.exercise_data['name'].replace("_", ""))
+        #self.m_history.add(urlobj.href)
+        #self.show_help_on_current()
     def _show_docviewer(self):
         if 'docviewer' not in self.box_dict:
             self.box_dict['docviewer'] = DocViewer(self.m_app.handle_href)
@@ -525,27 +586,27 @@ class MainWin(gnome.ui.App, cfg.ConfigUtils):
         self._show_docviewer()
         self.box_dict['docviewer'].source(html)
     def history_back(self, *v):
-        if self.m_viewer == 'docviewer':
-            self.m_history.set_adj_of_current(
-              self.box_dict['docviewer'].get_vadjustment().get_value())
+        #if self.m_viewer == 'docviewer':
+        #    self.m_history.set_adj_of_current(
+        #      self.box_dict['docviewer'].get_vadjustment().get_value())
         self.m_history.back()
         self.m_history.lock()
         url, adj = self.m_history.get_current()
         self.m_app.handle_href(url, display_docfile_set_adj=0)
-        if self.m_viewer == 'docviewer':
-            self.box_dict['docviewer'].get_vadjustment().set_value(adj)
+        #if self.m_viewer == 'docviewer':
+        #    self.box_dict['docviewer'].get_vadjustment().set_value(adj)
         self.m_history.unlock()
     def history_forward(self, *v):
-        if self.m_viewer == 'docviewer':
-            self.m_history.set_adj_of_current(
-              self.box_dict['docviewer'].get_vadjustment().get_value())
+        #if self.m_viewer == 'docviewer':
+        #    self.m_history.set_adj_of_current(
+        #      self.box_dict['docviewer'].get_vadjustment().get_value())
         self.m_history.forward()
         self.m_history.lock()
         url, adj = self.m_history.get_current()
         self.m_app.handle_href(url, display_docfile_set_adj=0)
-        if self.m_viewer == 'docviewer':
-            self.box_dict['docviewer'].get_vadjustment().set_value(adj)
-        self.m_history.unlock()
+        #if self.m_viewer == 'docviewer':
+        #    self.box_dict['docviewer'].get_vadjustment().set_value(adj)
+        #self.m_history.unlock()
     def history_reload(self, *v):
         self.m_history.lock()
         self.m_app.handle_href(self.m_history.get_current()[0])
@@ -559,6 +620,10 @@ class MainWin(gnome.ui.App, cfg.ConfigUtils):
         n = utils.exercise_name_to_module_name(teacher.m_exname)
         self.box_dict[teacher.m_exname] = globals()[n].Gui(teacher, self)
         self.main_box.pack_start(self.box_dict[teacher.m_exname])
+        if self.m_viewer:
+            self.box_dict[self.m_viewer].hide()
+        self.box_dict[n].show()
+        self.m_viewer = n
     def on_key_press_event(self, widget, event):
         if event.type == gtk.gdk.KEY_PRESS:
             for s in self.m_key_bindings:
@@ -575,27 +640,6 @@ class MainWin(gnome.ui.App, cfg.ConfigUtils):
             self.g_config_window.show()
     def on_zoom_in(self, *v):
         self.box_dict['docviewer'].m_htmlwidget.g_view.zoom_in()
-    def on_watch_toolbar_style(self, s):
-        assert self.get_string('gui/toolbar_style') in ('text', 'icons', 'both')
-
-        self.g_ui_manager.get_widget('/Toolbar').set_style({
-            'text': gtk.TOOLBAR_TEXT,
-            'icons': gtk.TOOLBAR_ICONS,
-            'both': gtk.TOOLBAR_BOTH}[self.get_string('gui/toolbar_style')])
-    def on_watch_toolbar_visible(self, s):
-        self.g_ui_manager.get_widget('/Menubar/ViewMenu/ToggleToolbarVisible').set_active(self.get_bool('gui/toolbar_visible'))
-        if self.get_bool('gui/toolbar_visible'):
-            self.g_ui_manager.get_widget('/Toolbar').show()
-        else:
-            self.g_ui_manager.get_widget('/Toolbar').hide()
-    def on_watch_navbar_visible(self, widget):
-        self.g_ui_manager.get_widget('/Menubar/ViewMenu/ToggleNavButtonsVisible').set_active(self.get_bool('gui/navbar_visible'))
-        if self.get_bool('gui/navbar_visible'):
-            self.g_ui_manager.get_widget('/Toolbar/BrowserGoBack').show()
-            self.g_ui_manager.get_widget('/Toolbar/BrowserGoForward').show()
-        else:
-            self.g_ui_manager.get_widget('/Toolbar/BrowserGoBack').hide()
-            self.g_ui_manager.get_widget('/Toolbar/BrowserGoForward').hide()
     def quit_program(self, w=None):
         self.m_app.quit_program()
         gtk.main_quit()
@@ -607,6 +651,45 @@ class MainWin(gnome.ui.App, cfg.ConfigUtils):
     def close_musicviewer(self, widget=None):
         self.g_musicviewer_window.destroy()
         self.g_musicviewer_window = None
+    def enter_test_mode(self, *v):
+        self.m_action_groups['NotExit'].set_sensitive(False)
+        self.g = self.box_dict[self.m_viewer].g_notebook.get_nth_page(0)
+        self.box_dict[self.m_viewer].g_notebook.get_nth_page(0).reparent(self.main_box)
+        self.box_dict[self.m_viewer].g_notebook.hide()
+        self.box_dict[self.m_viewer].enter_test_mode()
+    def exit_test_mode(self, *v):
+        self.m_app.m_test_mode = False
+        self.m_action_groups['NotExit'].set_sensitive(True)
+        box = gtk.VBox()
+        self.box_dict[self.m_viewer].g_notebook.insert_page(box, gtk.Label(_("Practise")), 0)
+        self.g.reparent(box)
+        self.box_dict[self.m_viewer].g_notebook.show()
+        self.box_dict[self.m_viewer].g_notebook.get_nth_page(0).show()
+        self.box_dict[self.m_viewer].g_notebook.set_current_page(0)
+        self.box_dict[self.m_viewer].exit_test_mode()
+        # rebuild the menus in case we hav passed a test.
+        self.create_practise_and_test_menu()
+
+def handle_lesson_id_crash(lessonfile_manager):
+    has_crash = False
+    for lesson_id in lessonfile_manager.iterate_duplicated_lesson_id():
+        has_crash = True
+        if splash_win:
+            splash_win.hide()
+        dlg = ResolveLessonIdCrashDlg(lessonfile_manager, lesson_id)
+        ret = dlg.run()
+        if ret == gtk.RESPONSE_OK:
+            for b in dlg.radio_dict.values():
+                if not b.get_active():
+                    f = lessonfile.LessonIdParser(b.get_data('fn'))
+                    f.new_lesson_id()
+        elif ret == gtk.RESPONSE_CANCEL:
+            lessonfile_manager.ignore_files_with_lesson_id(lesson_id)
+        dlg.destroy()
+    if splash_win:
+        splash_win.show()
+    if has_crash:
+        lessonfile_manager.parse()
 
 if splash_win:
     splash_win.show_progress("running gnome.program_init")
@@ -621,15 +704,21 @@ locale.setlocale(locale.LC_NUMERIC, "C")
 cfg.set_bool('config/no_random', bool(options.no_random))
 
 if splash_win:
-    splash_win.show_progress("creating MainWin")
+    splash_win.show_progress("creating LessonfileManager")
+lessonfile_manager = lessonfile.LessonFileManager()
+
+handle_lesson_id_crash(lessonfile_manager)
 
 if splash_win:
-    splash_win.destroy()
+    splash_win.show_progress("creating MainWin")
 
 def start_app(prefix, datadir):
-    w = MainWin(options, datadir)
+    w = MainWin(options, datadir, lessonfile_manager)
     w.show()
     w.post_constructor()
+    if splash_win:
+        splash_win.destroy()
+
     import gtk
     gtk.main()
 
