@@ -15,20 +15,12 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-__exercise_data__ = {
-    'exercise_name': 'rhythm',
-    'stock-icon': 'solfege-rhythm',
-    'menupath': '/Menubar/FileMenu/MiscMenu',
-    'name': _("_Rhythm"),
-}
-import exercise_setup
-exercise_setup.register_exercise(__exercise_data__)
-
 import gtk, gnome.ui, gobject
 import widgets
 import abstract, const, gu, mpd, soundcard
 import random, os
 import cfg
+import lessonfile
 
 RHYTHMS = ("c4", "c8 c8", "c16 c16 c16 c16", "c8 c16 c16",
            "c16 c16 c8", "c16 c8 c16", "c8. c16", "c16 c8.",
@@ -39,13 +31,13 @@ RHYTHMS = ("c4", "c8 c8", "c16 c16 c16 c16", "c8 c16 c16",
            )
 
 
-class Teacher(abstract.Teacher):
-    exercise_data = __exercise_data__
+class Teacher(abstract.LessonbasedTeacher):
     OK = 0
     ERR_PICKY = 1
     ERR_NO_ELEMS = 2
-    def __init__(self, exname, app, config):
-        abstract.Teacher.__init__(self, exname, app, config)
+    def __init__(self, exname, app):
+        abstract.Teacher.__init__(self, exname, app)
+        self.lessonfileclass = lessonfile.IdByNameLessonfile
     def new_question(self):
         """returns:
                Teacher.ERR_PICKY : if the question is not yet solved and the
@@ -414,6 +406,15 @@ class Gui(abstract.Gui):
         self.m_t.play_question()
         self.g_first_rhythm_button.grab_focus()
     def on_start_practise(self):
+        for n in range(22):
+            self.m_t.set_bool('rhythm_element_%i' % n, 
+                              n in self.m_t.m_P.header.rhythm_elements)
+        self.m_t.set_int('num_beats', self.m_t.m_P.header.num_beats)
+        self.m_t.set_int('bpm', self.m_t.m_P.header.bpm)
+        self.m_t.set_int('countin_perc', self.m_t.m_P.header.countin_perc)
+        self.m_t.set_int('rhythm_perc', self.m_t.m_P.header.rhythm_perc)
+        self.m_t.set_int('count_in', self.m_t.m_P.header.count_in)
+        self.m_t.set_bool('not_start_with_rest', self.m_t.m_P.header.not_start_with_rest)
         self.g_rhythm_viewer.flash(_("Click 'New' to begin."))
         self.g_new.grab_focus()
     def on_end_practise(self):
