@@ -130,7 +130,12 @@ class AbstractQuestionNameTable(gtk.Table, cfg.ConfigUtils):
             self.m_x = 0
             self.m_y = self.m_y + 1
     def grab_focus_first_button(self):
-        self.get_children()[-1].grab_focus()
+        v = self.get_children()
+        v.reverse()
+        for c in v:
+            if c.get_property("sensitive"):
+                c.grab_focus()
+                return
 
 class QuestionNameButtonTable(AbstractQuestionNameTable):
     def __init__(self, exname):
@@ -178,6 +183,11 @@ class QuestionNameCheckButtonTable(AbstractQuestionNameTable):
         AbstractQuestionNameTable.__init__(self, teacher.m_exname)
         self.m_t = teacher
     def ask_for_names_changed(self, *v):
+        self.m_t.m_P.m_random.reset()
+        if not self.m_name_list:
+            # If m_name_list is empty, it means that no lesson file is
+            # yet loaded. Just return.
+            return
         if self._ignore_watch > 0:
             return
         for question in self.m_t.m_P.m_questions:
