@@ -33,18 +33,15 @@ import gethomedir
 import configureoutput
 
 # version 1, the first version was introduced with Solfege 2.0
-statistics_version = 1
+# version 2 was introduced in the newxercisesetup changes
+statistics_version = 2
 
-STATISTICS_SAVE_DIR = os.path.join(gethomedir.get_home_dir(), '.solfege')
+STATISTICS_SAVE_DIR = os.path.join(gethomedir.get_home_dir(), '.solfege',
+    'statistics')
 
 if not os.path.exists(STATISTICS_SAVE_DIR):
-    for ver in '1.9', '1.4':
-        if os.path.exists(os.path.join(os.path.expanduser('~'), '.solfege%s' % ver)):
-            shutil.copytree(os.path.join(os.path.expanduser('~'), '.solfege%s' % ver),
-                            STATISTICS_SAVE_DIR)
-            break
     if not os.path.exists(STATISTICS_SAVE_DIR):
-        os.mkdir(STATISTICS_SAVE_DIR)
+        os.makedirs(STATISTICS_SAVE_DIR)
     f = file(os.path.join(STATISTICS_SAVE_DIR, 'statistics_version'), 'w')
     print >> f, statistics_version
     f.close()
@@ -103,10 +100,6 @@ class AbstractStatistics:
                         'today'   : self.m_today_stat,
                         'last7'   : self.m_last7_stat,
                         'total'   : self.m_total_stat}
-        self.m_savepath = os.path.join(STATISTICS_SAVE_DIR, self.m_t.m_exname)
-        if usable_statistics:
-            if not os.path.exists(self.m_savepath):
-                os.mkdir(self.m_savepath)
     def _add(self, question, answer):
         for D in [self.m_session_stat, self.m_today_stat,
                   self.m_last7_stat, self.m_total_stat]:
@@ -130,6 +123,9 @@ class AbstractStatistics:
                 A[k][n] = A[k][n] + B[k][n]
         return A
     def load_statistics(self, datadir):
+        """
+        datadir is the directory where the statistics files are stored.
+        """
         if not usable_statistics:
             return
         for filename in os.listdir(datadir):
@@ -214,8 +210,9 @@ class AbstractStatistics:
 class Statistics(AbstractStatistics):
     def __init__(self, teacher):
         AbstractStatistics.__init__(self, teacher)
-        self.load_statistics(self.m_savepath)
+        self.load_statistics(self.m_savepath)#FIXME newexercisesetup update
     def save_data(self):
+        raise "NewExerciseSetup update"
         if not usable_statistics:
             return
         if self.m_session_stat:
@@ -228,8 +225,21 @@ class LessonStatistics(AbstractStatistics):
     def __init__(self, teacher):
         AbstractStatistics.__init__(self, teacher)
         self.m_cur_collection = self.m_cur_file = None
-        self.lessonfile_changed(self.m_t.get_string('lessoncollection'),
-                                self.m_t.get_string('lessonfile'))
+    def enter_test_mode(self):
+        """
+        We assume that the program has saved and emptied the data dict
+        before calling this function.
+        """
+        #FIXME nuke STATISTICS_SAVE_DIR??
+        self.m_savepath = os.path.join(gethomedir.get_home_dir(), '.solfege',
+                'testresults', self.m_t.m_P.header.lesson_id)
+    def exit_test_mode(self):
+        """
+        We assume that the program has saved and emptied the data dict
+        before calling this function.
+        """
+        self.m_savepath = os.path.join(gethomedir.get_home_dir(), '.solfege',
+                'statistics', self.m_t.m_P.header.lesson_id)
     def get_label_style(self):
         return self.m_t.m_P.header.labelformat
     def lessonfile_changed(self, new_collection, new_file):
@@ -246,56 +256,50 @@ class LessonStatistics(AbstractStatistics):
         self.m_cur_file = new_file
         if not usable_statistics:
             return
+        self.m_savepath = os.path.join(STATISTICS_SAVE_DIR,
+                self.m_t.m_P.header.lesson_id)
         self.create_statistics_dir()
 
         # if the lessonfile has changed, we have to
         # delete all statistics just to be save.
         if self.get_hash_of_statistics() != self.get_hash_of_lessonfile():
-            p = os.path.join(self.m_savepath,
-                             self.m_cur_collection,
-                             self.m_cur_file)
-            if os.path.isfile("%s_hash" % p):
-                os.remove("%s_hash" % p)
-            for f in os.listdir(p):
-                os.remove(os.path.join(p, f))
-        self.load_statistics(os.path.join(self.m_savepath,
-                                          self.m_cur_collection,
-                                          self.m_cur_file))
+            if os.path.isfile("%s_hash" % self.m_savepath):
+                os.remove("%s_hash" % self.m_savepath)
+            for f in os.listdir(self.m_savepath):
+                os.remove(os.path.join(self.m_savepath, f))
+        self.load_statistics(self.m_savepath)
+    def store_test_passed(self):
+        f = open(os.path.join(self.m_savepath, "passed"), 'w')
+        f.write(str(self.get()))
+        f.close()
     def save_data(self):
         if not usable_statistics:
+            print "Warning: not saving statistics because usable_statistics is False"
             return
         if self.m_session_stat == {}:
             return
+        if not os.path.isdir(self.m_savepath):
+            os.makedirs(self.m_savepath)
         #save the hashvalue for the lessonfile this statistics was made with
-        f = open(os.path.join(self.m_savepath,
-                              self.m_cur_collection,
-                              '%s_hash' % self.m_cur_file), 'w')
+        f = open(os.path.join(self.m_savepath, "..",
+            '%s_hash' % self.m_t.m_P.header.lesson_id), 'w')
         f.write(str(self.get_hash_of_lessonfile()))
         f.close()
         f = open(os.path.join(self.m_savepath,
-                              self.m_cur_collection,
-                              self.m_cur_file,
                               str(int(time.time()))), 'w')
         pickle.dump(self.m_session_stat, f)
         f.close()
     def create_statistics_dir(self):
-        dir = os.path.join(self.m_savepath, self.m_cur_collection)
-        if not os.path.exists(dir):
-            os.mkdir(dir)
-
-        dir = os.path.join(dir, self.m_cur_file)
-        if not os.path.exists(dir):
-            os.mkdir(dir)
+        if not os.path.exists(self.m_savepath):
+            os.mkdir(self.m_savepath)
     def get_hash_of_statistics(self):
-        # !wrong function name!
+        # FIXME !wrong function name!
         # get the hash for the content of the lessonfile that was used last
         # time statistics was saved
-        if os.path.isfile(os.path.join(self.m_savepath,
-                                       self.m_cur_collection,
-                                       '%s_hash' % self.m_cur_file)):
-            s = open(os.path.join(self.m_savepath,
-                                         self.m_cur_collection,
-                                         '%s_hash' % self.m_cur_file)).read()
+        if os.path.isfile(os.path.join(self.m_savepath, "..",
+            '%s_hash' % self.m_t.m_P.header.lesson_id)):
+            s = open(os.path.join(self.m_savepath, "..",
+                  '%s_hash' % self.m_t.m_P.header.lesson_id)).read()
             try:
                 return int(s)
             except:
@@ -312,11 +316,11 @@ class LessonStatistics(AbstractStatistics):
                 return question.get_name()
         return key
 
-class IntervalStatistics(Statistics):
+class IntervalStatistics(LessonStatistics):
     def key_to_pretty_name(self, key):
         return utils.int_to_intervalname(key, 1, 1)
 
-class HarmonicIntervalStatistics(Statistics):
+class HarmonicIntervalStatistics(LessonStatistics):
     def key_to_pretty_name(self, key):
         return utils.int_to_intervalname(key, 1, 0)
 
